;
; ACEBETA.INC for ACE BETA 3
;  made by Hideaki Omuro
;

;
; RAM Locations
;
;Name         Address Len  Meaning
;   
KEY_0        = $8000 ;Byte Scancode of last key, 0 if gotten
KEY_1        = $8001 ;Byte Scancode of key down now
KEY_2        = $8002 ;Byte Same as $8001, $FF if more than one key is down
KEY_STAT     = $8004 ;Byte Bit 2 set = key down now
LAST_KEY     = $8006 ;Byte Last key pressed
CONTRAST     = $8008 ;Byte Contrast ($00 to $1F)
CURSOR_POS   = $800C ;Word Text position for word operations
CURSOR_ROW   = $800C ;Byte Text cursor row
CURSOR_COL   = $800D ;Byte Text cursor column
BUSY_COUNTER = $8026 ;Byte Counter for busy indicator
BUSY_BITMAP  = $8027 ;Byte Bitmap for busy indicator
OP1          = $8028 ;  11 OP1
OP2          = $8033 ;  11 OP2
OP3          = $803E ;  11 OP3
OP4          = $8049 ;  11 OP4
OP5          = $8054 ;  11 OP5
OP6          = $805F ;  11 OP6
TEXT_MEM     = $808F ; 128 text memory
GRAF_CURS    = $8215 ;Word Cursor position for graphics
CURSOR_X     = $8215 ;Byte X value of graphics cursor
CURSOR_Y     = $8216 ;Byte Y value of graphics cursor
APD_BUF      = $8228 ; 768 APD screen buffer ** DO NOT ACCESS **
_IY_TABLE    = $8528 ; N/A Where IY usually points
GRAPH_MEM    = $88B8 ; 768 Graphics Buffer
TEXT_MEM2    = $8BDF ; 128 Secondary text memory
WIN_START    = $8C8F ;Byte Number of first row in window
WIN_END      = $8C90 ;Byte Number of last row in window
VAT_START    = $FE6E ; N/A Start of VAT

;
; Variable Regions
;
SH_BUF       = $9000 ;  2k buffer

;
; ACE shell calls
;
; This is a list of functions within ACE.

FILL         = $8D50 ; Fill BC bytes with A, starting at HL
GET_RAND     = $8D53 ; Get random # 0-255
PAUSE        = $8D56 ; Wait for [ENTER]
HL_DECI      = $8D59 ; Convert decimal HL to a string, returns HL->string
GRB_COPY     = $8D5C ; Copy graphics buffer to LCD
GRB_FILL     = $8D5F ; Fill graphics buffer with A
DIRECT_KEY   = $8D62 ; put state of down,left,right,up,Y=,2nd,mode,del into A
DISP_DELAY   = $8D65 ; delay for LCD write

FIND_PIXEL   = $8D68 ; *** not implemented yet
SPRITE       = $8D6B ; *** not implemented yet
LINK_SEND    = $8D6E ; *** not implemented yet
LINK_UPDATE  = $8D71 ; *** not implemented yet
LINK_GET     = $8D74 ; *** not implemented yet

EXIT_TIOS    = $8D77 ; Exit to TI-OS
FIND_INIT    = $8D7A ; Initialize Variable Find (call before searching a var)
FIND_NEXT    = $8D7D ; Return HL->vat entry,DE->prog,OP1=name of next var
KHAND        = $8D80 ; key handler w/ 2nd/alpha/contrast/off

CR_GRBCopy   = GRB_COPY ; duplicate definition
CR_KHAND     = KHAND    ; duplicate definition

;----------------------------------------------------------------------------
;[ Macro Definitions ]
;----------------------------------------------------------------------------
#DEFINE LD_HL_MHL LD A,(HL)\ INC HL\ LD H,(HL)\ LD L,A
#DEFINE CP_HL_DE  PUSH HL\ OR A\ SBC HL,DE\ POP HL
#DEFINE CP_HL_BC  PUSH HL\ OR A\ SBC HL,BC\ POP HL

;----------------------------------------------------------------------------
;[ ROM call vectors ]
;----------------------------------------------------------------------------
; This is a list of functions inside the TI-82 ROM.

#define ROM_CALL(x) CALL x

TX_CHARPUT  = TEXT_MEM2
M_CHARPUT   = TEXT_MEM2 + 3
D_ZT_STR    = TEXT_MEM2 + 6
D_ZM_STR    = TEXT_MEM2 + 9
CLEAR_TEXT  = TEXT_MEM2 + 12
CLEAR_LCD   = TEXT_MEM2 + 15
GET_KEY     = TEXT_MEM2 + 18
STORE_KEY   = TEXT_MEM2 + 21
FIND_VAT    = TEXT_MEM2 + 24
UNPACK_HL   = TEXT_MEM2 + 27
CREATE_PRGM = TEXT_MEM2 + 30
DEL_MEM     = TEXT_MEM2 + 33
INSERT_MEM  = TEXT_MEM2 + 36
MEM_FREE    = TEXT_MEM2 + 39
DEL_TMP     = TEXT_MEM2 + 42
CREATE_TMP  = TEXT_MEM2 + 45
KEY_HAND    = $8D4D

;
; Ports
;
P_LINK       = $00   ; Link port                                     (TI-85)
P_KEYBOARD   = $01   ; Port used to access keys                      (TI-85)
P_ROM        = $02   ; Rom page switching and other things
P_STATUS     = $03   ; Used to get/set status of varius hardware
P_INT        = $04   ; Interrupt speed (don't change!!! the display needs it)
P_LCD_CON    = $10   ; Control port for display controller           (TI-83)
P_LCD_DATA   = $11   ; Data port for display controller              (TI-83)

;
; Misc
;
#DEFINE TEXT_START RES 1,(IY+0Dh) \ RES 2,(IY+0Dh) \ ROM_CALL(CLEAR_DISP) \ LD HL,0000h \ LD (CURSOR_POS),HL
#DEFINE TEXT_END SET 2,(IY+0Dh) \ SET 1,(IY+0Dh)

G_NONE   = $00
G_DOWN   = $01
G_LEFT   = $02
G_RIGHT  = $03
G_RIGTH  = $03
G_UP     = $04
G_ENTER  = $09
G_PLUS   = $0A
G_MINUS  = $0B
G_TIMES  = $0C
G_DIV    = $0D
G_CARROT = $0E
G_CLEAR  = $0F
G_NEG    = $11
G_3      = $12
G_6      = $13
G_9      = $14
G_PARENR = $15
G_TAN    = $16
G_VARS   = $17
G_PERIOD = $19
G_2      = $1A
G_5      = $1B
G_8      = $1C
G_PARENL = $1D
G_COS    = $1E
G_PRGM   = $1F
G_STAT   = $20
G_0      = $21
G_1      = $22
G_4      = $23
G_7      = $24
G_COMMA  = $25
G_SIN    = $26
G_MATRIX = $27
G_XTO    = $28
G_ON     = $29
G_STO    = $2A
G_LN     = $2B
G_LOG    = $2C
G_SQR    = $2D
G_INVE   = $2E
G_MATH   = $2F
G_ALPHA  = $30
G_GRAPH  = $31
G_TRACE  = $32
G_ZOOM   = $33
G_WINDOW = $34
G_YEDIT  = $35
G_2ND    = $36
G_MODE   = $37
G_DEL    = $38

K_RIGTH   =   1 ; RIGHT     RIGHT
K_RIGHT   =   1 ; RIGHT     RIGHT
K_LEFT    =   2 ; LEFT      LEFT
K_UP      =   3 ; UP        UP
K_DOWN    =   4 ; DOWN      DOWN
K_ENTER   =   5 ; ENTER     ENTER
K_CLEAR   =   6 ; CLEAR     CLEAR
K_DEL     =   7 ; DEL       DEL
K_INS     =   8 ; DEL       INS
K_RCL     =   9 ; STO>      RCL
K_ENTRY   =  10 ; ENTER     ENTRY
K_BOL     =  11 ; LEFT      BOL
K_EOL     =  12 ; RIGHT     EOL
K_PRGM    =  45 ; PRGM      PRGM
K_ZOOM    =  46 ; ZOOM      ZOOM
K_DRAW    =  47 ; PRGM      DRAW
K_PLOT    =  48 ; Y=        STAT PLOT
K_MATH    =  49 ; MATH      MATH
K_TEST    =  50 ; MATH      TEST
K_VARS    =  52 ; VARS      VARS
K_MEM     =  53 ; +         MEM
K_MATRIX  =  54 ; MATRIX    MATRIX
K_STAT    =  55 ; STAT      STAT
K_Y_VARS  =  56 ; VARS      Y-VARS
K_ANGLE   =  57 ; MATRIX    ANGLE
K_LIST    =  58 ; STAT      LIST
K_CALC    =  59 ; TRACE     CALC
K_QUIT    =  64 ; MODE      QUIT
K_LINK    =  65 ; X,T,theta LINK
K_GRAPH   =  69 ; GRAPH     GRAPH
K_MODE    =  70 ; MODE      MODE
K_WINDOW  =  73 ; WINDOW    WINDOW
K_YEDIT   =  74 ; Y=        Y=
K_TABLE   =  75 ; GRAPH     TABLE
K_TBLSET  =  76 ; WINDOW    TblSet
K_TRACE   =  93 ; TRACE     TRACE
K_PLUS    = 129 ; +         +
K_MINUS   = 130 ; -         -
K_TIMES   = 131 ; *         *
K_DIV     = 132 ; /         /
K_POWER   = 133 ; ^         ^
K_LPAREN  = 134 ; (         (
K_RPAREN  = 135 ; )         )
K_LBRACK  = 136 ; *         [
K_RBRACK  = 137 ; -         ]
K_STO     = 139 ; STO>      STO>
K_COMMA   = 140 ; ,         ,
K_NEG     = 141 ; (-)       (-)
K_PERIOD  = 142 ; .         .
K_0       = 143 ; 0         0
K_1       = 144 ; 1         1
K_2       = 145 ; 2         2
K_3       = 146 ; 3         3
K_4       = 147 ; 4         4
K_5       = 148 ; 5         5
K_6       = 149 ; 6         6
K_7       = 150 ; 7         7
K_8       = 151 ; 8         8
K_9       = 152 ; 9         9
K_EE      = 153 ; ,         EE
K_SEMI    = 154 ; 0         ;
K_A       = 155 ; MATH      A
K_B       = 156 ; MATRIX    B
K_C       = 157 ; PRGM      C
K_D       = 158 ; X^-1      D
K_E       = 159 ; SIN       E
K_F       = 160 ; COS       F
K_G       = 161 ; TAN       G
K_H       = 162 ; ^         H
K_I       = 163 ; X^2       I
K_J       = 164 ; ,         J
K_K       = 165 ; (         K
K_L       = 166 ; )         L
K_M       = 167 ; /         M
K_N       = 168 ; LOG       N
K_O       = 169 ; 7         O
K_P       = 170 ; 8         P
K_Q       = 171 ; 9         Q
K_R       = 172 ; *         R
K_S       = 173 ; LN        S
K_T       = 174 ; 4         T
K_U       = 175 ; 5         U
K_V       = 176 ; 6         V
K_W       = 177 ; -         W
K_X       = 178 ; STO>      X
K_Y       = 179 ; 1         Y
K_Z       = 180 ; 2         Z
K_XTO     = 181 ; X,T,theta X,T,theta
K_PI      = 182 ; ^         Pi
K_INVERSE = 183 ; X^-1      X^-1
K_SIN     = 184 ; SIN       SIN
K_ASIN    = 185 ; SIN       SIN^-1
K_COS     = 186 ; COS       COS
K_ACOS    = 187 ; COS       COS^-1
K_TAN     = 188 ; TAN       TAN
K_ATAN    = 189 ; TAN       TAN^-1
K_SQR     = 190 ; X^2       X^2
K_SQRT    = 191 ; X^2       SQRT
K_LN      = 192 ; LN        LN
K_EX      = 193 ; LN        e^X
K_LOG     = 194 ; LOG       LOG
K_TENP    = 195 ; LOG       10^X
K_ANS     = 198 ; (-)       ANS
K_COLON   = 199 ; .         :
K_QUEST   = 203 ; (-)       ?
K_QUOTE   = 204 ; +         "
K_THETA   = 205 ; 3         THETA
K_LBRACE  = 237 ; (         {
K_RBRACE  = 238 ; )         }
K_ABS     = 244 ; X^-1      ABS
K_L1      = 245 ; 1         L1
K_L2      = 246 ; 2         L2
K_L3      = 247 ; 3         L3
K_L4      = 248 ; 4         L4
K_L5      = 249 ; 5         L5
K_L6      = 250 ; 6         L6
;K_n       = 251 ; 9         n
K_Vn1     = 252 ; 8         Vn-1
K_Un1     = 253 ; 7         Un-1

START_ADDR = $9880
.ORG START_ADDR       ; All ACE Programs start at address $9880.

; The Mystery of the .END directive...
.END ; End is needed in every single program, and TASM will have a fit over
     ; it, and this works, so here it is.
